% VerySmartPointer is a wrapper around SmartPointers
%
% This is part of the OTB Matlab Toolbox
% 
% author: Rik Bellens


classdef VerySmartPointer < handle
    
    properties (SetAccess = private)
        SmartPointer
        Pointer
    end
    
    methods
        function obj = VerySmartPointer(in)
            if ischar(in)
                obj.Pointer = javaMethod([in '_New'],['InsightToolkit.' in]);
                obj.SmartPointer = obj.Pointer.GetPointer;
            elseif isa(in,'VerySmartPointer')
                obj.Pointer = in.Pointer;
                obj.SmartPointer = in.SmartPointer;
            else
                classname = char(in.getClass());
                classname = classname((length('class ')+1):end);
                if strfind(classname,'_Pointer')
                    obj.Pointer = in;
                    obj.SmartPointer = obj.Pointer.GetPointer;
                else
                    obj.SmartPointer = in;
                    classname = [classname '_Pointer'];
                    obj.Pointer = javaObject(classname,obj.SmartPointer);
                end
            end
            obj.Pointer.SetReferenceCount(obj.Pointer.GetReferenceCount+1);                
        end
        function delete(obj)
            obj.SmartPointer.SetReferenceCount(obj.SmartPointer.GetReferenceCount-1);
        end
        function disp(obj)
            i = InsightToolkit.itkIndent(0);
            s = InsightToolkit.itkStringStream;
            obj.SmartPointer.Print(s.GetStream,i);
            disp(s.GetString);
        end
        function out = Call(obj,method,varargin)
            for i=1:length(varargin)
                if isa(varargin{i},'VerySmartPointer')
                    varargin{i} = varargin{i}.SmartPointer;
                end
            end
            if nargout==1
                out = javaMethod(method,obj.SmartPointer,varargin{:});
                if isa(out,'InsightToolkit.itkObject')
                    out = VerySmartPointer(out);
                end
            else
                javaMethod(method,obj.Pointer,varargin{:});
            end
        end
        function [namespace, type, suffix] = nameparts(obj)
            classname = char(obj.Pointer.getClass);
            pattern = '^class InsightToolkit.(?<namespace>[a-z]*)(?<type>\w+)_Pointer$';
            r = regexp(classname,pattern,'names');
            namespace = r(1).namespace;
            type = r(1).type;
            pattern = '^(?<suffix>[A-Z0-9]+)(?<type>[a-z]\w+)$';
            r = regexp(type(end:-1:1),pattern,'names');
            type = r(1).type(end:-1:1);
            suffix = r(1).suffix(end:-1:1);
        end
        function b = isimage(obj)
            [namespace, type] = nameparts(obj);
            b = false;
            if strcmp(type,'Image')||strcmp(type,'VectorImage')
                b = true;
            end
        end
        function s = suffix(obj)
            [namespace, type, s] = nameparts(obj);
            switch type
                case 'Image'
                    s = ['I' s];
                case 'VectorImage'
                    s = ['VI' s];
            end
        end
    end
end