% img = readimage(filename)
%
% reads an image from file
%
% This is part of the OTB Matlab Toolbox
% 
% author: Rik Bellens

function img = readimage(filename)

reader = VerySmartPointer('otbImageFileReaderIUS2');
reader.Call('SetFileName',filename)
reader.Call('UpdateOutputInformation')

io = reader.Call('GetImageIO');

ndimensions = io.Call('GetNumberOfDimensions');
componenttype = char(io.Call('GetComponentTypeAsString',io.Call('GetComponentType')));
pixeltype = char(io.Call('GetPixelTypeAsString',io.Call('GetPixelType')));
ncomponents = io.Call('GetNumberOfComponents');

switch componenttype
    case {'double'}
        suffix = 'D';
    case {'float'}
        suffix = 'F';
    case {'unsigned_char'}
        suffix = 'UC';
    case {'signed_char'}
        suffix = 'SC';
    case {'unsigned_int'}
        suffix = 'UI';
    case {'signed_int'}
        suffix = 'SI';
    case {'unsigned_short'}
        suffix = 'US';
    case {'signed_short'}
        suffix = 'SS';
    case {'unsigned_long'}
        suffix = 'UL';
    case {'signed_long'}
        suffix = 'SL';
    otherwise 
        error(['unknown component type: ' componenttype]);
end

switch pixeltype
    case {'scalar'}
        suffix = ['I' suffix];
    case {'vector'}
        suffix = 'VIF';
end

suffix = [suffix int2str(ndimensions)];

readername = ['otbImageFileReader' suffix];
reader = VerySmartPointer(readername);
reader.Call('SetFileName',filename);
img = reader.Call('GetOutput');
reader.Call('Update');

